@@ -4,6 +4,9 @@
 
 //! Core components and resources to use Heron
 
+use core::any::Any;
+use std::sync::Arc;
+
 use bevy::ecs::schedule::ShouldRun;
 use bevy::prelude::*;
 
@@ -13,11 +16,10 @@ pub use gravity::Gravity;
 pub use layers::{CollisionLayers, PhysicsLayer};
 pub use physics_time::PhysicsTime;
 pub use step::{PhysicsStepDuration, PhysicsSteps};
-pub use velocity::{Acceleration, AxisAngle, Velocity};
+pub use velocity::{Acceleration, AxisAngle, Damping, Velocity};
 
 mod constraints;
 mod events;
-pub mod ext;
 mod gravity;
 mod layers;
 mod physics_time;
@@ -66,14 +68,13 @@ impl Plugin for CorePlugin {
             .register_type::<PhysicMaterial>()
             .register_type::<Velocity>()
             .register_type::<Acceleration>()
+            .register_type::<Damping>()
             .register_type::<RotationConstraints>()
             .register_type::<CollisionLayers>()
             .register_type::<SensorShape>()
             .add_system_to_stage(CoreStage::First, PhysicsSteps::update.system())
             .add_stage_before(CoreStage::PostUpdate, crate::stage::ROOT, {
-                Schedule::default()
-                    .with_run_criteria(should_run.system())
-                    .with_stage(crate::stage::UPDATE, SystemStage::parallel())
+                Schedule::default().with_stage(crate::stage::UPDATE, SystemStage::parallel())
             });
     }
 }
@@ -91,6 +92,37 @@ pub fn should_run(
     }
 }
 
+/// An opaque type representing a custom collision shape.
+///
+/// It will accept [`Any`] type, but the implementation is abstract, and
+/// what actual values will work depends on the backend, so see
+/// the relevant backend documentation to learn what types are accepted here.
+#[derive(Clone)]
+pub struct CustomCollisionShape(Arc<dyn Any + Send + Sync>, &'static str);
+
+impl CustomCollisionShape {
+    /// Create a new [`CustomCollisionShape`] that wraps some value.
+    pub fn new<T: Any + Send + Sync>(shape: T) -> Self {
+        Self(Arc::new(shape), std::any::type_name::<T>())
+    }
+
+    /// Check if the stored value is of type `T`, and give a reference to it
+    /// if the type matches.
+    /// Will return [`None`] if the type of the stored value does not match `T`.
+    #[must_use]
+    pub fn downcast_ref<T: Any>(&self) -> Option<&T> {
+        self.0.downcast_ref()
+    }
+}
+
+impl core::fmt::Debug for CustomCollisionShape {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        f.debug_tuple("CustomCollisionShape")
+            .field(&format_args!("{}", &self.1))
+            .finish()
+    }
+}
+
 /// Components that defines the collision shape of a rigid body
 ///
 /// The collision shape will be attached to the [`RigidBody`] of the same entity.
@@ -107,7 +139,9 @@ pub fn should_run(
 ///         .insert(RigidBody::Dynamic) // Create a dynamic rigid body
 ///         .insert(CollisionShape::Sphere { radius: 1.0 }); // Attach a collision shape
 /// }
+/// ```
 #[derive(Debug, Clone, Reflect)]
+#[non_exhaustive]
 pub enum CollisionShape {
     /// A sphere (or circle in 2d) shape defined by its radius
     Sphere {
@@ -163,6 +197,39 @@ pub enum CollisionShape {
         /// inner `Vec`, any other element will be ignored.
         heights: Vec<Vec<f32>>,
     },
+
+    /// A Cone shape, like a traffic cone, with a circular base
+    ///
+    /// This shape is exclusive to the 3d API, you must enable the "3d" flag to use it.
+    /// For the 2d equivalent, look at [`Sphere`](CollisionShape::Sphere).
+    #[cfg(dim3)]
+    Cone {
+        /// Half of the height from the base of the cone to the top point
+        half_height: f32,
+        /// The radius of the base circle
+        radius: f32,
+    },
+
+    /// A Cylinder shape, it is a cirlce with a height
+    ///
+    /// This shape is exclusive to the 3d API, you must enable the "3d" flag to use it.
+    /// For the 2d equivalent, look at [`Sphere`](CollisionShape::Sphere).
+    #[cfg(dim3)]
+    Cylinder {
+        /// Half of the height from the base of the cylinder to the top
+        half_height: f32,
+        /// The radius of the base circle
+        radius: f32,
+    },
+
+    /// A Custom shape, the actual shape is abstracted, and will be determined
+    /// by a corresponding backend depending on the implementation details
+    ///
+    /// See [`CustomCollisionShape`] for more info.
+    Custom {
+        /// The custom collision shape to use
+        shape: CustomCollisionShape,
+    },
 }
 
 impl Default for CollisionShape {
